from unittest.mock import patch

import pytest
from infoblox_client import objects

from gso.products.product_types.iptrunk import Iptrunk
from test.workflows import (
    assert_complete,
    assert_lso_interaction_success,
    extract_state,
    run_workflow,
)


@pytest.mark.workflow()
@patch("gso.services.infoblox.find_network_by_cidr")
@patch("gso.services.infoblox.find_v6_host_by_fqdn")
@patch("gso.services.infoblox.find_host_by_fqdn")
@patch("gso.workflows.iptrunk.validate_iptrunk.execute_playbook")
def test_validate_router_success(
    mock_validate_iptrunk,
    mock_find_host_by_fqdn,
    mock_find_v6_host_by_fqdn,
    mock_find_network_by_cidr,
    faker,
    data_config_filename,
    iptrunk_subscription_factory,
):
    #  Mock value setup
    subscription_id = iptrunk_subscription_factory()
    trunk = Iptrunk.from_subscription(subscription_id).iptrunk
    mock_find_network_by_cidr.side_effects = [
        objects.Network(connector=None, ipv4addrs=[trunk.iptrunk_ipv4_network]),
        objects.NetworkV6(connector=None, ipv6addrs=[trunk.iptrunk_ipv6_network]),
    ]
    mock_find_host_by_fqdn.side_effect = [
        objects.HostRecord(
            connector=None,
            aliases=[trunk.iptrunk_sides[0].iptrunk_side_node.router_fqdn],
            comment=subscription_id,
            ipv4addrs=[
                objects.IPv4(
                    ipv4addr=str(trunk.iptrunk_ipv4_network[0]),
                    configure_for_dhcp=False,
                    mac="00:00:00:00:00:00",
                    ip=str(trunk.iptrunk_ipv4_network[0]),
                    host=f"{trunk.iptrunk_sides[0].iptrunk_side_ae_iface}.{trunk.iptrunk_sides[0].iptrunk_side_node.router_fqdn}",
                ),
            ],
            name=trunk.iptrunk_sides[0].iptrunk_side_node.router_fqdn,
        ),
        objects.HostRecord(
            connector=None,
            aliases=[trunk.iptrunk_sides[1].iptrunk_side_node.router_fqdn],
            comment=subscription_id,
            ipv4addrs=[
                objects.IPv4(
                    ipv4addr=str(trunk.iptrunk_ipv4_network[1]),
                    configure_for_dhcp=False,
                    mac="00:00:00:00:00:00",
                    ip=str(trunk.iptrunk_ipv4_network[1]),
                    host=f"{trunk.iptrunk_sides[1].iptrunk_side_ae_iface}.{trunk.iptrunk_sides[1].iptrunk_side_node.router_fqdn}",
                ),
            ],
            name=trunk.iptrunk_sides[1].iptrunk_side_node.router_fqdn,
        ),
    ]

    mock_find_v6_host_by_fqdn.side_effect = [
        objects.HostRecordV6(
            connector=None,
            aliases=[trunk.iptrunk_sides[0].iptrunk_side_node.router_fqdn],
            comment=subscription_id,
            ipv4addrs=[
                objects.IPv6(
                    ipv6addr=str(trunk.iptrunk_ipv6_network[0]),
                    configure_for_dhcp=False,
                    mac="00:00:00:00:00:00",
                    ip=str(trunk.iptrunk_ipv6_network[0]),
                    host=f"{trunk.iptrunk_sides[0].iptrunk_side_ae_iface}.{trunk.iptrunk_sides[0].iptrunk_side_node.router_fqdn}",
                ),
            ],
            name=trunk.iptrunk_sides[0].iptrunk_side_node.router_fqdn,
        ),
        objects.HostRecordV6(
            connector=None,
            aliases=[trunk.iptrunk_sides[1].iptrunk_side_node.router_fqdn],
            comment=subscription_id,
            ipv4addrs=[
                objects.IPv6(
                    ipv4addr=str(trunk.iptrunk_ipv6_network[1]),
                    configure_for_dhcp=False,
                    mac="00:00:00:00:00:00",
                    ip=str(trunk.iptrunk_ipv6_network[1]),
                    host=f"{trunk.iptrunk_sides[1].iptrunk_side_ae_iface}.{trunk.iptrunk_sides[1].iptrunk_side_node.router_fqdn}",
                ),
            ],
            name=trunk.iptrunk_sides[1].iptrunk_side_node.router_fqdn,
        ),
    ]

    #  Run workflow
    initial_router_data = [{"subscription_id": subscription_id}]
    result, process_stat, step_log = run_workflow("validate_iptrunk", initial_router_data)

    state = extract_state(result)
    subscription_id = state["subscription_id"]

    result, step_log = assert_lso_interaction_success(result, process_stat, step_log)

    assert_complete(result)

    subscription = Iptrunk.from_subscription(subscription_id)

    assert subscription.status == "active"
    assert mock_validate_iptrunk.call_count == 1
    assert mock_find_host_by_fqdn.call_count == 2
    assert mock_find_v6_host_by_fqdn.call_count == 2
    assert mock_find_network_by_cidr.call_count == 2
